\name{RODBC-internal}
\alias{odbcColData}
\alias{odbcNumRows}
\alias{odbcNumCols}
\alias{odbcUpdate}
\alias{odbcCaseFlag}
\alias{odbcSpecialColumns}
\alias{sqltablecreate}
\alias{sqlwrite}
\alias{odbcTableExists}
\alias{odbcTypeInfo}
\alias{odbcClearResults}
\alias{odbcValidChannel}

\title{Internal RODBC functions}
\description{
  Internal RODBC functions
}
\usage{
odbcColData(channel)
odbcNumRows(channel)
odbcNumCols(channel)

odbcUpdate(channel, query, data, names, test = FALSE, verbose = FALSE,
           nastring = NULL)
odbcCaseFlag(channel)
odbcSpecialColumns(channel, table)
sqlwrite(channel, tablename, mydata, test = FALSE, fast = TRUE,
         nastring = NULL, verbose = FALSE)
sqltablecreate(tablename, coldata, keys=-1)
odbcTableExists(channel, tablename, abort=TRUE)
odbcTypeInfo(channel, type)
odbcClearResults(channel)
odbcValidChannel(channel)
}
\keyword{internal}

\eof
\name{RODBC}
\alias{sqlCopy}
\alias{sqlCopyTable}

\title{ODBC Database Connectivity}
\usage{
sqlCopy(channel, query, destination, destchannel = channel, verbose = FALSE,
        errors = TRUE, \dots)
sqlCopyTable(channel, srctable, desttable, destchannel = channel,
             verbose = FALSE, errors = TRUE)
}
\arguments{
  \item{channel, destchannel}{connection handle as returned by
    \code{\link{odbcConnect}}.}
  \item{query}{any valid SQL statement}
  \item{destination, srctable, desttable}{a
    database table name accessible from the connected dsn.  This should be
    either a character string or a character vector of length 1.}
  \item{verbose}{Display statements as they are sent to the server?}
  \item{errors}{if TRUE halt and display error, else return -1.}
  \item{\dots}{additional arguments to be passed to \code{\link{sqlSave}}.}
}
\description{
  \code{RODBC} implements ODBC database connectivity with compliant
  databases where drivers exist on the host system.
}
\details{
  Two groups
  of commands are provided.  \code{odbc*} commands implement relatively
  low level access to the odbc functions of similar name.
  \code{sql*} commands are higher level constructs to read, save, copy and
  manipulate data between data frames and sql tables.  Up to 16 connections
  can be open at once to any combination of dsn/hosts.
  Writing columns is limited to 255 characters wide.

  \code{sqlCopy} as is like \code{\link{sqlQuery}},
  but saves the output of \code{query} in table \code{destination} on
  channel \code{destchannel}. [Not yet implemented.]

  \code{sqlCopyTable}
  copies the structure of \code{srctable} to \code{desttable} on dsn
  \code{destchannel}.  This is within the limitations of the ODBC lowest
  common denominator.  More precise control is possible via
  \code{sqlQuery}.

  A few functions
  try to cope with the peculiar way the Excel ODBC driver handles names.
}
\value{
  See \code{\link{sqlGetResults}}.
}
\seealso{
  \code{\link{odbcConnect}}, \code{\link{sqlFetch}},
  \code{\link{sqlSave}}, \code{\link{sqlTables}}, \code{\link{odbcGetInfo}}
}
\author{
  Michael Lapsley and Brian Ripley
}
\examples{
\dontrun{library(RODBC)
data(USArrests)
channel <- odbcConnect("test", "", "") # userId and password
sqlSave(channel, USArrests, rownames = "State", verbose = TRUE)
options(dec=".") # this is the default decimal point
sqlQuery(channel, "select State, Murder from USArrests where rape > 30 order by Murder")
sqlFetch(channel, "USArrests", rownames = "State")
sqlDrop(channel, "USArrests")
odbcClose(channel)
}}
\keyword{IO}
\keyword{database}

\eof
\name{RODBCtables}
\alias{sqlPrimaryKeys}
\alias{sqlColumns}
\alias{sqlDrop}
\alias{sqlClear}

\title{Operations on Tables in ODBC databases}
\description{
  Operations on tables in ODBC databases.
}
\usage{
sqlColumns(channel, sqtable, errors = FALSE, as.is = TRUE, special = FALSE)
sqlPrimaryKeys(channel, sqtable, errors = FALSE, as.is = TRUE)
sqlDrop(channel, sqtable, errors = TRUE)
sqlClear(channel, sqtable, errors = TRUE)
}
\arguments{
  \item{channel}{connection handle as returned by \code{\link{odbcConnect}}.}
  \item{sqtable}{a database table name accessible from the connected
    dsn. This should be either a character string or a character vector
    of length 1.}
  \item{errors}{if TRUE halt and display error, else return -1}
  \item{as.is}{as in \code{\link{sqlGetResults}}.}
  \item{special}{return only the column(s) needed to specify a row
    uniquely.  Depending on the database, there might be none.}
}
\details{
  \code{sqlClear} deletes the content of the table \code{sqtable}.  No
  confirmation is requested.

  \code{sqlDrop} removes the table \code{sqtable}.  No confirmation
  is requested.

  \code{sqlColumns} and \code{sqlPrimaryKeys} return
  information as data frames.  Note that the column names often contain
  underscores and are invalid in \R unless quoted.  The column names are
  not constant across ODBC versions so the data should be accessed by
  column number.  The argument \code{special} to \code{sqlColumns}
  returns the columns needed to specify a row uniquely.  This is
  intended to form the basis of a WHERE clause for updates (see
  \code{\link{sqlUpdate}}.
}
\value{
  A data frame on success, or character/numeric on error depending on the
  \code{errors} parameter.  If no data is returned, either a zero-row
  data frame or an error. (For example, if there are no primary keys or
  special column(s) in this table
  an empty data frame is returned, but if primary keys are not supported
  by the DBMS, an error code results.)
}
\seealso{
  \code{\link{odbcConnect}}, \code{\link{sqlQuery}}, \code{\link{sqlFetch}},
  \code{\link{sqlSave}}, \code{\link{sqlTables}}, \code{\link{odbcGetInfo}}
}
\author{
  Michael Lapsley and Brian Ripley
}
\examples{
\dontrun{## example results from MySQL
data(USArrests)
channel <- odbcConnect("test", "", "") # userId and password
sqlDrop(channel, "USArrests", errors = FALSE) # precautionary
sqlSave(channel, USArrests)
sqlColumns(channel, "USArrests")
sqlColumns(channel, "USArrests", special = TRUE)
sqlPrimaryKeys(channel, "USArrests")
sqlColumns(channel, "USArrests")
##   Table_cat Table_schema Table_name Column_name Data_type Type_name
## 1                         USArrests    rownames        12   varchar
## 2                         USArrests      murder         8    double
## 3                         USArrests     assault         4   integer
## 4                         USArrests    urbanpop         4   integer
## 5                         USArrests        rape         8    double
##   Column_size Buffer_length Decimal_digits Num_prec_radix Nullable Remarks
## 1         255           255           <NA>           <NA>        0
## 2          22             8             31             10        1
## 3          11             4              0             10        1
## 4          11             4              0             10        1
## 5          22             8             31             10        1
sqlColumns(channel, "USArrests", special = TRUE)
##   Scope Column_name Data_type Type_name Precision Length Scale
## 1     2    rownames        12   varchar         0      0     0
##   Pseudo_column
## 1             1
sqlPrimaryKeys(channel, "USArrests")
##   Table_qualifer Table_owner Table_name Column_name Key_seq Pk_name
## 1           <NA>        <NA>  USArrests    rownames       1 PRIMARY
sqlClear(channel, "USArrests")
sqlDrop(channel, "USArrests")
odbcClose(channel)
}}
\keyword{IO}
\keyword{database}

\eof
\name{odbc}
\alias{odbcQuery}
\alias{odbcTables}
\alias{odbcColumns}
\alias{odbcPrimaryKeys}
\alias{odbcFetchRows}

\title{Low-level ODBC functions}
\description{
  \R functions which talk directly to the ODBC interface.
}
\usage{
odbcTables(channel)

odbcColumns(channel, table)
odbcPrimaryKeys(channel, table)

odbcQuery(channel, query)
odbcFetchRows(channel, max = 0, buffsize = 1000, nullstring = NA,
              believeNRows = TRUE) 
}
\arguments{
  \item{channel}{connection handle as returned by
    \code{\link{odbcConnect}()} of class \code{"RODBC"}.}
  \item{query}{any valid SQL statement}
  \item{table}{a database table name accessible from the connected dsn.
    This can be either a character string or an (unquoted) symbol.}
  \item{max}{limit on the number of rows to fetch, with 0 indicating no limit.}
  \item{buffsize}{the number of records to be transferred at a time.}
  \item{nullstring}{
    character string to be used when reading \code{SQL_NULL_DATA} items
    from the database.
  }
  \item{believeNRows}{logical.  Is the number of rows returned by the
    ODBC connection believable?  Not true for Oracle, apparently.}
}
\details{
  \code{odbcFetchRows} returns a data frame of the pending
  rowset in \code{$data} limited to \code{max} rows if
  \code{max} is greater than 0.  \code{buffsize} may be increased from the
  default of 1000 rows for increased performance in a large dataset.
  This only has an effect with servers that do not return the number of rows
  affected by a query e.g. MS Access, MS SQLServer.
}
\value{
  Most return \code{1} on success and \code{-1} on failure,
  indicating that a message is waiting for \code{\link{odbcGetErrMsg}}.
  \code{odbcFetchRows} may return \code{-2} indicating \code{"No Data"},
  the message that would be returned by \code{\link{odbcGetErrMsg}}.
}
\seealso{
  \code{\link{sqlQuery}}, \code{\link{odbcConnect}},
  \code{\link{odbcGetErrMsg}}.
}
\author{
  Michael Lapsley and Brian Ripley
}
\keyword{IO}
\keyword{database}

\eof
\name{odbcClose}
\alias{odbcClose}
\alias{odbcCloseAll}
\alias{close.RODBC}
\title{ODBC Close Connections}
\description{
  Close connections to ODBC databases.
}
\usage{
odbcClose(channel)
\method{close}{RODBC}(con, \dots)
odbcCloseAll()
}
\arguments{
  \item{channel, con}{RODBC connection object returned by \code{odbcConnect}.}
  \item{\dots}{additional arguments passed from the generic.}
}
\details{
  \code{odbcClose} cleans up and frees resources. It is also the method for the
  generic function \code{close}.

  \code{odbcCloseAll} closes all open channels,
}
\value{
  Function \code{odbcClose} returns invisibly a logical indicating if it
  succeeded.
}
\author{
  Michael Lapsley, Brian Ripley
}
\seealso{
  \code{\link{odbcConnect}}
}
\keyword{IO}
\keyword{database}

\eof
\name{odbcConnect}
\alias{odbcConnect}
\alias{odbcDriverConnect}
\alias{odbcReConnect}
\alias{print.RODBC}
#ifdef windows
\alias{odbcConnectAccess}
\alias{odbcConnectDbase}
\alias{odbcConnectExcel}
#endif
\title{ODBC Open Connections}
\description{
  Open connections to ODBC databases.
}
\usage{
odbcConnect(dsn, uid = "", pwd = "", case = "nochange",
            believeNRows = TRUE)
odbcDriverConnect(connection, case = "nochange", believeNRows = TRUE)
odbcReConnect(channel, case, believeNRows)
#ifdef windows

odbcConnectAccess(access.file, uid = "", pwd = "")
odbcConnectDbase(dbf.file)
odbcConnectExcel(xls.file)
#endif
}
\arguments{
  \item{dsn}{character string. A registered data source name.}
  \item{uid, pwd}{UID and password for authentication (if required).}
  \item{connection}{character string. See your ODBC documentation for the
    format.}
  \item{case}{Controls case changes for different RDBMS engines.}
  \item{channel}{RODBC connection object returned by \code{odbcConnect}.}
  \item{believeNRows}{logical.  Is the number of rows returned by the
    ODBC connection believable?  Not true for Oracle, apparently.}
#ifdef windows
  \item{access.file, dbf.file, xls.file}{file of an appropriate type.}
#endif
}
\details{
  \code{odbcConnect} establishes a connection to the dsn, and
  \code{odbcDriverConnect} allows a more flexible specification via a
  connection string.  \code{odbcConnect} uses the connection string
  \code{"DSN=dsn;UID=uid;PWD=pwd"}.  See the examples for other
  uses of connection strings.

#ifdef windows
  Under the Windows GUI, specifying an incomplete \code{connection}, for
  example \code{""}, will bring up a dialog box to complete the
  information required.

#endif
  For databases that translate table and column names the case must be
  set as appropriate.  Allowable values are \code{"nochange"},
  \code{"toupper"} and \code{"tolower"} as well as the names of databases
  where the behaviour is known to us (currently \code{"mysql"} (which
  maps to lower case on Windows but not on Linux),
  \code{"postgresql"} (lower), \code{"oracle"} (upper) and
  \code{"msaccess"}) (nochange).

  Function \code{odbcReConnect} re-connects to a database using the
  settings of an existing (and presumably now closed) channel object.
  Arguments \code{case} and \code{believeNRows} are taken from the
  object, but can be overridden by supplying those arguments.
#ifdef windows

  \code{odbcConnectAccess}, \code{odbcConnectDbase} and
  \code{odbcConnectExcel} are convenience wrappers to generate
  connection strings for those file types.  The files given can be
  relative to the \R working directory or absolute paths (and it seems
  that the drivers will also look relative to the user's home
  directory).  \bold{Note:} they will only work with English versions of
  the MicroSoft drivers, which may or may not be installed in other locales.
#endif
}
\value{
  A non-negative
  integer which is used as handle if no error occurred, -1 otherwise.
  A successful return has class \code{"RODBC"}, and attribute
  \code{"connection.string"} giving the full ODBC connection string.
}
\author{
  Michael Lapsley, Brian Ripley
}
\seealso{
  \code{\link{odbcClose}}, \code{\link{sqlQuery}}, \code{\link{odbcGetInfo}}
}
\examples{\dontrun{
#ifdef unix
# MySQL
channel <- odbcConnect("test", uid="ripley", pwd="secret")
# PostgreSQL
channel <- odbcConnect("pg", uid="ripley", pwd="secret", case="postgresql")
#endif
#ifdef windows
# interactive specification
channel <- odbcDriverConnect("")

# MySQL on Windows -- MySQL maps to lower case on Windows only
channel <- odbcConnect("testdb", uid="ripley", case="tolower")

# Access
channel <- odbcConnect("testacc") # if this was set up as a DSN
channel2 <- odbcConnectAccess("test.mdb", uid="ripley")

# Excel
channel <- odbcConnect("bdr.xls") # if this was set up as a DSN
channel2 <-
 odbcDriverConnect("DRIVER=Microsoft Excel Driver (*.xls);DBQ=C:\\bdr\\hills.xls")
channel3 <- odbcConnectExcel("bdr.xls")
#endif

# re-connection
odbcCloseAll()
channel <- odbcReConnect(channel) # must re-assign as the data may well change
}}
\keyword{IO}
\keyword{database}

\eof
\name{odbcErrors}
\alias{odbcGetErrMsg}
\alias{odbcClearError}

\title{RODBC Error-Handling Utility Functions}
\description{
  Utility functions for examining and clearing error messages which have
  been collected during low-level ODBC calls.
}
\usage{
odbcGetErrMsg(channel)
odbcClearError(channel)
}
\arguments{
  \item{channel}{connection handle as returned by
    \code{\link{odbcConnect}()} of class \code{"RODBC"}.}  
}
\value{
  \code{odbcGetErrMsg} returns a (possibly zero-length)
  character vector of pending messages.

  \code{odbcClearError} returns nothing, invisibly.
}
\seealso{
  \code{\link{odbcQuery}}, \code{\link{odbcConnect}}
}
\author{
  Michael Lapsley
}
\keyword{IO}
\keyword{database}

\eof
\name{odbcGetInfo}
\alias{odbcGetInfo}
\title{Request Information on an ODBC Connection}
\description{
  Request information on an ODBC connection.
}
\usage{
odbcGetInfo(channel)
}
\arguments{
  \item{channel}{connection handle as returned by
    \code{\link{odbcConnect}()} of class \code{"RODBC"}.}
}
\value{
  A character string giving information on the database and ODBC driver
  in use on the connection \code{channel}.
}
\author{
  Brian Ripley
}
\examples{\dontrun{odbcGetInfo()
## MySQL returned
[1] "MySQL version 3.23.53-max-nt. Driver ODBC version 02.50"
## MS Access returned
[1] "ACCESS version 04.00.0000. Driver ODBC version 03.51"
}}
\keyword{IO}
\keyword{database}

\eof
\name{sqlFetch}
\alias{sqlFetch}

\title{Reading Tables from ODBC Databases}
\description{
  Read a table of an ODBC database into a data frame.
}
\usage{
sqlFetch(channel, sqtable, ..., colnames = FALSE, rownames = TRUE)
}
\arguments{
  \item{channel}{connection handle returned by \code{\link{odbcConnect}}.}
  \item{sqtable}{a database table name accessible from the connected dsn.
    This should be either a character string or a character vector of
    length 1.}
  \item{\dots}{additional arguments to be passed to
    \code{\link{sqlGetResults}}.}
  \item{colnames}{logical: retrieve column names from first row of table?}
  \item{rownames}{either logical or character.
    If logical, retrieve row names from the first column
    (\code{rownames}) in the table?  If character, the column name to
    retrieve them.}
}
\details{
  \code{sqlFetch} retrieves the the entire contents of the table
  \code{sqtable}.  Rownames and column names are restored as indicated.

  It tries to cope with the peculiar way the Excel ODBC
  driver handles table names.

  Useful additional parameters to pass to \code{sqlGetResults} include
  \describe{
    \item{\code{max}:}{limit on the number of rows to fetch, with
      0 (the default) indicating no limit.}
    \item\code{{nullstring}:}{character string to be used when reading
      \code{SQL_NULL_DATA} character items from the database.}
    \item{\code{na.strings}:}{character string(s) to be mapped to
      \code{NA} when reading character data.}
    \item{\code{as.is}:}{as in \code{\link{sqlGetResults}}.}
    \item{\code{dec}:}{The character for the decimal place to be assumed
      when converting character columns.}
  }
}
\value{
  A data frame on success, or a character or numeric error code (see
  \code{\link{sqlQuery}}).
}
\seealso{
  \code{\link{sqlSave}}, \code{\link{sqlQuery}},
  \code{\link{odbcConnect}}, \code{\link{odbcGetInfo}}
}
\author{
  Michael Lapsley and Brian Ripley
}
\examples{
\dontrun{library(RODBC)
data(USArrests)
channel <- odbcConnect("test", "", "") # userId and password
sqlSave(channel, USArrests, verbose = TRUE)
sqlFetch(channel, "USArrests") # get the lot
sqlDrop(channel, "USArrests") 
odbcClose(channel)
}}
\keyword{IO}
\keyword{database}

\eof
\name{sqlQuery}
\alias{sqlQuery}
\alias{sqlGetResults}

\title{Query an ODBC Database}
\description{
  Submit an SQL query to an ODBC database, and retrieve the results.
}
\usage{
sqlQuery(channel, query, errors = TRUE, ...)
sqlGetResults(channel, as.is = FALSE,  errors = FALSE,
              max = 0, buffsize = 1000,
              nullstring = NA, na.strings = "NA",
              believeNRows = TRUE, dec = getOption("dec"))
}
\arguments{
  \item{channel}{connection handle as returned by \code{\link{odbcConnect}}.}
  \item{query}{any valid SQL statement}
  \item{errors}{if TRUE halt and display error, else return -1}
  \item{\dots}{additional arguments to be passed to \code{sqlGetResults}.}
  \item{as.is}{which (if any) character columns should be converted,
    as in \code{\link{read.table}}?  See the details.}
  \item{max}{limit on the number of rows to fetch, with 0 indicating no limit.}
  \item{buffsize}{the number of rows to be transferred at a time.}
  \item{nullstring}{
    character string to be used when reading \code{SQL_NULL_DATA}
    character items from the database.
  }
  \item{na.strings}{
    character string(s) to be mapped to \code{NA} when reading character
    data.
  }
  \item{believeNRows}{logical.  Is the number of rows returned by the
    ODBC connection believable?  This might already be set to false when
    the channel was opened, and can that setting cannot be overriden.}
  \item{dec}{The character for the decimal place to be assumed when
    converting character columns.}
}
\details{
  \code{sqlQuery} is the workhorse function of RODBC.  It sends the SQL
  statement \code{query} to the server, using connection \code{channel}
  returned by \code{\link{odbcConnect}}.

  SQL beginners should note that the term `Query' includes
  any valid SQL statement including table creation, alteration, updates
  etc as well as SELECTs. The \code{sqlQuery} command is a convenience
  wrapper that calls first \code{\link{odbcQuery}} and then
  \code{sqlGetResults}. If finer-grained control is needed, for example
  over the number of rows fetched, these functions should be called
  directly or additional arguments passed to \code{sqlQuery}.

  \code{sqlGetResults} is a mid-level function.  It should be called
  after a call to \code{\link{odbcQuery}} and used to retrieve waiting
  results into a data frame. Its main use is with \code{max} set to non
  zero it will retrieve the result set in batches with repeated calls.
  This is useful for very large result sets which can be subjected to
  intermediate processing.  It makes use of the setting of
  \code{options("dec")} to determine the character used in setting
  decimal points.  This is set to \code{"."} when RODBC is initialized
  if it is not already set.

  Where possible \code{sqlGetResults} transfers data directly: this
  happens for \code{double}, \code{real}, \code{integer} and
  \code{smallint} columns in the table.  All other SQL data types are
  converted to character strings (of length up to 256), and then converted
  by \code{\link{type.convert}} as controlled by the \code{as.is}
  argument.  Note that almost all callers set \code{as.is = TRUE}.
  
  Using \code{buffsize} will yield a
  marginal increase in speed if set to more than the maximum number of
  rows where \code{believeNRows = FALSE}.
}
\value{
  A data frame (possibly with 0 rows) on success.
  If \code{errors = TRUE}, a character vector of error message(s),
  otherwise error code \code{-1} (general, call
  \code{\link{odbcGetErrMsg}} for details) or \code{-2} (no data, which
  may not be an error as some SQL commands do return no data).
}
\seealso{
  \code{\link{odbcConnect}}, \code{\link{sqlFetch}},
  \code{\link{sqlSave}}, \code{\link{sqlTables}}, \code{\link{odbcQuery}}
}
\author{
  Michael Lapsley and Brian Ripley
}
\examples{
\dontrun{library(RODBC)
data(USArrests)
channel <- odbcConnect("test", "", "") # userId and password
sqlSave(channel, USArrests, rownames = "State", verbose = TRUE)
options(dec=".") # this is the default decimal point
## note case of State, Murder, rape are DBMS-dependent.
sqlQuery(channel, "select State, Murder from USArrests where rape > 30 order by Murder")
odbcClose(channel)
}}
\keyword{IO}
\keyword{database}

\eof
\name{sqlSave}
\alias{sqlSave}
\alias{sqlUpdate}

\title{Write a Data Frame to a Table in an ODBC Database}
\description{
  Write or update a table in an ODBC database.
}
\synopsis{
sqlSave(channel, dat, tablename = NULL, append = FALSE, rownames = TRUE,
        colnames = FALSE, verbose = FALSE, oldstyle = FALSE,
        safer = TRUE, addPK = FALSE, ...)
sqlUpdate(channel, dat, tablename = NULL, index = NULL,
          verbose = FALSE, test = FALSE, nastring = NULL, fast = TRUE)
}
\usage{
sqlSave(channel, dat, tablename = NULL, append = FALSE, rownames = TRUE,
        colnames = FALSE, verbose = FALSE, oldstyle = FALSE,
        safer = TRUE, addPK = FALSE, fast = TRUE, test = TRUE,
        nastring = NULL)

sqlUpdate(channel, dat, tablename = NULL, index = NULL,
          verbose = FALSE, test = FALSE, nastring = NULL, fast = TRUE)
}
\arguments{
  \item{channel}{connection handle returned by \code{\link{odbcConnect}}.}
  \item{dat}{a data frame}
  \item{tablename}{a
    database table name accessible from the connected dsn.  This should be
    either a character string or a character vector of length 1.}
  \item{index}{character.  Name(s) of index column(s) to be used.}
  \item{append}{logical.  Should data be appended to an existing table?}
  \item{rownames}{either logical or character.
    If logical, save the row names as the first column \code{rownames}
    in the table?  If character, the column name under which to save
    the rownames.}
  \item{colnames}{logical: save column names as the first row of table?}
  \item{verbose}{display statements as they are sent to the server?}

  \item{oldstyle}{logical. If false, attempt to use \code{sqlTypeInfo}
    to choose the types of columns when a table has to be created.  If
    true, create all columns as \code{varchar(255)}.}
  \item{safer}{logical. If true, create a non-existing table but only
    allow appends to an existing table. If false, allow \code{sqlSave}
    to attempt to delete all the rows of an existing table, or to drop it.}
  \item{addPK}{logical.  Should rownames (if included) be specified as a
    primary key?}
  \item{fast}{logical.  If false, write data a row at a time.  If true,
    use a parametrized INSERT INTO / UPDATE query to write all the data
    in one operation.}
  \item{test}{show what would be done, only.}
  \item{nastring}{character string to be used for writing \code{NA}s
    to the database.  The default, \code{NULL}, attempts to write
    a missing value as a database null.}
  \item{\dots}{any additional arguments are passed to \code{\link{sqlwrite}}.}
}
\details{
  \code{sqlSave} saves the data frame \code{dat} in the table
  \code{tablename}.  The table name is taken from tablename if given or
  the name of the data frame specified at \code{dat}.
  If the table exists and has the appropriate structure
  it is used, or else it is created anew.
  If \code{rownames = TRUE} the first column of the table will be the row
  labels with colname \code{rowname}:  \code{rownames}
  can also be a string giving the desired name (see example). \code{colnames}
  copied the column names into row 1.  This is intended for cases where
  case conversion alters the original column names and it is desired that
  they are retained.  Note that there are drawbacks to this approach:
  it presupposes that the rows will be returned in correct order;
  not always valid.  It will also cause numeric rows to be returned as factors.

  Argument \code{addPK = TRUE} causes the rownames to be marked as a
  primary key.  This is usually a good idea, and may allow updates to be
  done.  However, some DBMSs (e.g. Access) do not support primary keys,
  and the PostgreSQL ODBC driver generates internal memory corruption if
  this option is used.
  
  WARNING: \code{sqlSave(safer = FALSE)} uses the 'great white shark'
  method of testing tables (bite it and see).  The logic will
  unceremoniously DROP the table and create it anew with its own choice of
  column types in its attempt to find a writable solution.
  \code{test=TRUE} will not necessarily predict this behaviour.
  Attempting to write indexed columns or writing to pseudo-columns are
  less obvious causes of failed writes followed by a DROP. If your table
  structure is precious to you back it up.

  \code{sqlUpdate} updates the table where the rows already exist. Data
  frame \code{dat} should columns with names that map to (some of) the
  columns in the table.  It also needs to contain the column(s)
  specified by \code{index} which together identify the rows to be
  updated.  If \code{index = NULL}, the function tries to identify such
  rows.  First it looks for a primary key in the data frame, then for
  the column(s) that the database regards as the optimal for defining a
  row uniquely (these are returned by
  \code{\link{sqlColumns}(\dots, special=TRUE)}).  If
  there is no such column the rownames are used provided they are stored
  as column \code{"rownames"} in the table.

  The value of \code{nastring} is used for all the columns and no
  attempt is made to check if the column is nullable.  For all but the
  simplest applications it will be better to prepare a data frame with
  non-null missing values already substituted.
}
\value{
  1 invisibly for success (and failures cause errors).
}
\seealso{
  \code{\link{sqlFetch}}, \code{\link{sqlQuery}},
  \code{\link{odbcConnect}}, \code{\link{odbcGetInfo}}
}
\author{
  Michael Lapsley and Brian Ripley
}
\examples{
\dontrun{library(RODBC)
data(USArrests)
channel <- odbcConnect("test", "", "") # userId and password
sqlSave(channel, USArrests, verbose = TRUE, rownames = "State")
sqlFetch(channel, "USArrests", rownames = "State") # get the lot
foo <- cbind(state=row.names(USArrests), USArrests)[1:3, c(1,3)]
foo[1,2] <- 222
sqlUpdate(channel, foo, "USArrests")
sqlFetch(channel, "USArrests", rownames = "state", max = 5)
sqlDrop(channel, "USArrests") 
odbcClose(channel)
}}
\keyword{IO}
\keyword{database}

\eof
\name{sqlTables}
\alias{sqlTables}

\title{List Tables on an ODBC Database}

\description{
  List the tables on an ODBC database.
}
\usage{
sqlTables(channel, errors = FALSE, as.is = TRUE)
}
\arguments{
  \item{channel}{connection handle as returned by
    \code{\link{odbcConnect}}.}
  \item{errors}{if TRUE halt and display error, else return -1}
  \item{as.is}{as in \code{\link{sqlGetResults}}.}
}
\value{
  A data frame on success, or character/numeric
  on error depending on the \code{errors} parameter. (See
  \code{\link{sqlGetResults}} for further details.)

  The column names depend on the database, containing a column
  \code{table_name} in some mixture of cases (see the examples).
}
\seealso{
  \code{\link{sqlGetResults}}
}
\author{
  Michael Lapsley and Brian Ripley
}
\examples{
\dontrun{library(RODBC)
channel <- odbcConnect("test", "", "") # userId and password
sqlTables(channel, "USArrests")
## MySQL example
##   Table_qualifer Table_owner Table_name Table_type     Remarks
##1                             usarrests      TABLE MySQL table
## Microsoft Access example
##      TABLE_CAT TABLE_SCHEM        TABLE_NAME   TABLE_TYPE REMARKS
##1 C:\\bdr\\test        <NA> MSysAccessObjects SYSTEM TABLE    <NA>
##2 C:\\bdr\\test        <NA>          MSysACEs SYSTEM TABLE    <NA>
##3 C:\\bdr\\test        <NA>       MSysObjects SYSTEM TABLE    <NA>
##4 C:\\bdr\\test        <NA>       MSysQueries SYSTEM TABLE    <NA>
##5 C:\\bdr\\test        <NA> MSysRelationships SYSTEM TABLE    <NA>
##6 C:\\bdr\\test        <NA>             hills        TABLE    <NA>
##7 C:\\bdr\\test        <NA>         USArrests        TABLE    <NA>
odbcClose(channel)
}}
\keyword{IO}
\keyword{database}

\eof
\name{sqlTypeInfo}
\alias{sqlTypeInfo}

\title{Request Information about Column Types in an ODBC Database}
\description{
  Request information about column types in an ODBC database
}
\usage{
sqlTypeInfo(channel, type = "all", errors = TRUE, as.is = TRUE)
}
\arguments{
  \item{channel}{connection handle as returned by
    \code{\link{odbcConnect}}.}
  \item{type}{The types of columns about which information is
    requested. Possible values are \code{"all", "char", "varchar", "real",
      "double", "integer", "smallint", "timestamp"}.}
  \item{errors}{if TRUE halt and display error, else return -1}
  \item{as.is}{as in \code{\link{sqlGetResults}}.}
}
\details{
  \code{sqlTypeInfo} attempts to find the types of columns the database
  supports.  Not all ODBC drivers support this.  Where it is supported,
  it is used to decide what column types to create when creating a new
  table in the database.
}
\value{
  A data frame on success, or character/verbose
  on error depending on the \code{errors} parameter.  See
  \code{link{sqlGetResults}} for further details.
}
\seealso{
  \code{\link{sqlGetResults}}, \code{\link{odbcGetInfo}}
}
\author{
  Brian Ripley
}
\keyword{IO}
\keyword{database}

\eof
